# -*- coding: utf-8 -*-

# Project Library
from kinder.webservice.spyne_ws.exceptions import SpyneException
from kinder.core.declaration.models import DeclarationUnit
from kinder.core.children.models import (
    Children, DULTypeEnumerate, Delegate)
from kinder.core.dict.models import (
    HealthNeed, UnitKind)
from kinder.core.privilege.models import Privilege
from kinder.core.unit.models import Unit
from concentrator.models import (
    DelegatePerson, DocExtraInfo)
from concentrator.rules import *


class PrivilegeProxy(object):
    """
    Прокси класс для работы с привилегиями
    """

    def __init__(self, benefits):
        """
        benefits - list of Spyne Benefit instance
        """

        self._benefits = benefits

    @staticmethod
    def _get_benefit(benefit_info):

        benefit, benefit_type = benefit_info
        try:
            privilege = Privilege.objects.get(id=benefit)
        except Privilege.DoesNotExist:
            raise SpyneException(u'Льгота не найдена')

        return privilege

    def get(self):

        privileges = []

        for benefit, benefit_type in self._benefits:

            privileges.append(
                PrivilegeProxy._get_benefit(
                    (benefit, benefit_type)))

        return privileges


class ChildrenProxy(object):
    """
    Прокси класс для работы с Ребенком
    """

    def __init__(self, children, delegate, health_need):
        """
        children - Spyne DeclaredPersonData instance
        delegate - Spyne ApplicantData instance
        """

        self._delegate = delegate
        self._children = children
        self._health_need = health_need

    def _get_health_need(self):
        health_need = None
        if self._health_need:
            try:
                health_need = HealthNeed.objects.get(
                    id=self._health_need)
            except HealthNeed.DoesNotExist:
                raise SpyneException(u'Потребность по здоровью не найдена')

        return health_need

    def _get_sex(self):
        return SexTypeRule.get_system(self._children.Sex)

    def get(self):

        return Children(
            firstname=self._children.FirstName,
            surname=self._children.LastName,
            patronymic=self._children.MiddleName,
            snils=self._children.Snils,
            address_full=self._children.AddressResidence,
            reg_address_full=self._children.AddressRegistration,
            dul_type=DULTypeEnumerate.SVID,
            dul_series=self._children.BirthDocSeria,
            dul_number=self._children.BirthDocNumber,
            dul_date=self._children.BirthDocIssueDate,
            gender=self._get_sex(),
            date_of_birth=self._children.DateOfBirth,
            health_need=self._get_health_need(),
            zags_act_number=self._children.BirthDocActNumber,
            zags_act_place=self._children.BirthDocIssuer,
            birthplace=self._children.BirthPlace)


class DelegateProxy(object):
    """
    Прокси класс для работы с Представителем
    """

    def __init__(self, delegate):
        """
        delegate - Spyne ApplicantData instance
        """

        self._delegate = delegate

    def _get_doc_type(self):
        return DelegateDocTypeRule.get_system(self._delegate.DocType)

    def _get_delegate_type(self):
        return DelegateTypeRule.get_system(self._delegate.ApplicantType)

    def get(self):
        delegate = Delegate(
            firstname=self._delegate.FirstName,
            surname=self._delegate.LastName,
            patronymic=self._delegate.MiddleName,
            snils=self._delegate.Snils,
            dul_type_id=self._get_doc_type(),
            dul_series=self._delegate.DocSeria,
            dul_number=self._delegate.DocNumber,
            dul_date=self._delegate.DocIssueDate,
            dul_place=self._delegate.DocIssuerDepartmentCode,
            dul_issued_by=self._delegate.DocIssuerName,
            type=self._get_delegate_type(),
            email=self._delegate.Email,
            phones=self._delegate.PhoneNumber)

        delegate_extension = DelegatePerson(delegate=delegate,
                                            doc_type=int(self._delegate.DocType)
        )
        return delegate, delegate_extension


class DeclarationUnitProxy(object):
    """
    Прокси класс для работы с Желаемыми Учреждениями
    """

    def __init__(self, units, declaration):
        """
        units - list of spyne EduOrganization
        declaration - Django model instance
        """
        # Отбираем только, те у которых задан Code и приоритет,
        # остальные игнорируем.
        self._units = [unit for unit in units if unit.Code and unit.Priority]
        self._declaration = declaration

        if len([1 for u in self._units if u.Priority == 1]) > 1:
            raise SpyneException(u'Указано более одного приоритетного '
                                 u'желаемого учреждения')

    @staticmethod
    def _get_unit(spyne_unit):
        try:
            unit_id = int(spyne_unit.Code)
        except UnicodeEncodeError:
            raise SpyneException(u'Неверно передан идентификатор учреждения.'
                                 u'Проверьте корректность заполнения тега '
                                 u'<Code>')
        try:
            unit = Unit.objects.get(
                id=unit_id, kind__id=UnitKind.DOU)
        except Unit.DoesNotExist:
            # TODO: Что делать если не найдено хотя бы одно учреждение
            raise SpyneException(u'Учреждение не найдено')

        return unit

    def get_mo(self):
        if not self._units:
            raise SpyneException(u'Не указаны желаемые учреждения')
        current_mo = None
        for desired_unit in self._units:

            unit = self._get_unit(desired_unit)
            if current_mo and current_mo != unit.get_mo():
                raise SpyneException(u'Все ЖУ должны быть из одного МО')
            else:
                current_mo = unit.get_mo()

        return current_mo

    def get(self):
        from concentrator.webservice.helpers import get_declaration_units
        declaration_units = []
        existing_priorities = []

        # словарь unit_id -> ord, хранящихся в системе
        sys_unit_ord = dict(get_declaration_units(self._declaration))

        # для начала сортируем список учреждений:
        # - сначала по возрастанию приоритета из запроса, т.е. 1,2,2,...
        # - потом по возрастанию приоритета из системы. если доу новое и
        #     его нет среди желвемых, то проставляем большой приоритет 999
        # - потом по возрастанию Id учреждения
        self._units.sort(key=lambda u: (
            u.Priority, sys_unit_ord.get(int(u.Code), 999), u.Code))

        # ЖУ с приоритетом 1 обрабатываем однозначно. Остальным ЖУ
        # с приоритетом 2 проставляем приоритеты такие же как в системе,
        # так как по-другому их не возможно определить.
        for desired_unit in self._units:
            unit = self._get_unit(desired_unit)

            if desired_unit.Priority == 1:
                unit_order = desired_unit.Priority

                # только в случае если новое приоритетное доу есть среди желаемых,
                # выполняем подмену приоритетов
                if unit.id in sys_unit_ord:
                    tmp_order = sys_unit_ord.get(unit.id)
                    # Получаем Id Учреждения с концентратора,
                    # которому пользователь хочет проставить
                    # приоритет первый(unit), находим в системе
                    # текущее учреждение с приоритетом =1 (sys_first_unit_id),
                    # и проставляем учреждению unit приоритет =1,
                    # а учреждению sys_first_unit_id, тот приоритет(tmp_order),
                    # который сейчас учреждения unit.
                    for sys_first_unit_id, ord in sys_unit_ord.items():
                        if ord == 1:
                            sys_unit_ord.update([(sys_first_unit_id, tmp_order)])
                            break
            else:
                unit_order = sys_unit_ord.get(unit.id, None)

                # если доу нет среди текущих ЖУ заявки (новое доу - неприоритетное)
                # или приоритет уже содержится в списоке существующих
                # (новое доу - приоритетное),
                # то проставляем его сами по возрастанию id (см. сортировку выше)
                if unit_order is None or unit_order in existing_priorities:
                    if existing_priorities:
                        unit_order = max(existing_priorities) + 1
                    else:
                        unit_order = 1

            # добавляем приоритет в список существующих
            existing_priorities.append(unit_order)

            # в итоге получаем unit_order - приоритетс ЕПГУ
            # по необходимости скорректированный
            declaration_units.append(DeclarationUnit(
                unit=unit, ord=unit_order))

        return declaration_units


class DocExtraInfoProxy(object):

    def __init__(self, declaration, document_references):

        self._declaration = declaration
        self._document_references = document_references or []

    def save(self):

        for document_reference in self._document_references:

            doc_extra_info = DocExtraInfo(
                declaration=self._declaration,
                code=document_reference.Code)

            if document_reference.Name:
                doc_extra_info.name = document_reference.Name

            if document_reference.Description:
                doc_extra_info.description = document_reference.Description

            doc_extra_info.save()
